/* VirtuinoSE example: Client code to communicate with VirtuinoSE server code
 * Supported boards: Arduino Uno,Mega, Nano etc.. + Ethernet shiend
 * Created by Ilias Lamprou
 * Updated Noe 1 2018
 * More examples: http://virtuino.com/index.php/virtuino-se/20-virtuino-se-code-examples
 */

#include <SPI.h>
#include <Ethernet.h>
//#include <Ethernet2.h>    // enable this line to support Ethernet 2

//-----------------SETTINGS -------------------
const char* virtuinoServerIP = "192.168.1.150";    // The server IP address
int virtuinoServerPort = 8000;                     // The server port number
String virtuinoKEY="1234";                         // must be the same with the server Virtuino KEY
 
byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED};       // Set the ethernet shield mac address.
IPAddress ip(192, 168, 1, 50);                          // Set the ethernet shield ip address. 

//---------------------------------------------
unsigned long timeOfLastUpdate=0;

//==================================================== setup
//====================================================
void setup() {
  Serial.begin(9600);
  while (!Serial) {;}

  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    Ethernet.begin(mac, ip);     // try to congifure using IP address instead of DHCP:
  }
  delay(1000);
  Serial.println(Ethernet.localIP());
}



//==================================================== loop
//====================================================

void loop() {

   if (millis()-timeOfLastUpdate>10000) {    // update values to server every 10 seconds
    //----------- Add code here to read your sensor values
    int value1 = random(100);              // Replace with the value you want to send to virtuinoSE server

    //----------- Prepare the command data that the client has to send to VirtuinoSE server code   
    String commandBuffer="&KEY="+virtuinoKEY;   // Start a new command with the VirtuinoKey. If you don't want to use a key, start the command with an empty String
   
    commandBuffer+="&V0=?";                 // Command 1. Ask Virtuino server what is the value of memory V0
    commandBuffer+="&V1="+String(value1);   // Command 2. Send the value1 to memory V1
    commandBuffer+="&V2=?";                 // Command 3. Ask Virtuino server what is the value of memory V2
    
    String response=sendCommand(commandBuffer);  // send all the commands to virtuino server. Get the response
    Serial.println("Response: "+response);
    
    String valueOf_V0_asString = getValue(0,&response);          // get the value of V0 as string 
    if (valueOf_V0_asString.length()>0){                         
      float valueOf_V0= valueOf_V0_asString.toFloat();           // convert the text value to float or int
      Serial.println("valueOf_V0="+String(valueOf_V0));          // do something with value
    }

    String valueOf_V2_asString = getValue(2,&response);          // get the value of V2 as string 
    if (valueOf_V2_asString.length()>0){                         
      int valueOf_V2= valueOf_V2_asString.toInt();           // convert the text value to float or int
      Serial.println("valueOf_V2="+String(valueOf_V2));          // do something with value
    }

    timeOfLastUpdate=millis();
   }
   
}











//=============== Virtuino Client Library for Ethernet connection(included) ===========
//==============================================================
//==============================================================
/* Only the next two functions are nedded to communicate with Virtuino server code
 * Avoid to make changes to the code below  
 */

//==================================================== sendCommand
String sendCommand(String commandData){
  Serial.print("connecting to ");
  Serial.println(virtuinoServerIP);
  
  // Use WiFiClient class to create TCP connections
  EthernetClient client;
    if (!client.connect(virtuinoServerIP,virtuinoServerPort)) {
    Serial.println("connection failed");
    return "ERROR";
  }
  
  // Create a URI  and send the command to server
  client.print(String("GET /") + commandData+ "\r\n\r\n HTTP/1.1\r\n" +
               "Host: " + virtuinoServerIP + "\r\n" + 
               "Connection: close\r\n\r\n");
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println(">>> Client Timeout !");
      client.stop();
      return "Client Timeout";
    }
  }
  
  // Read all the lines of the reply from server and print them to Serial
  while(client.available()){
    String line = client.readStringUntil('\r');
    int dataStartIndex= line.indexOf('&');   // check if the response contains data
    if (dataStartIndex!=-1) {
      line = line.substring(dataStartIndex);
      client.flush();
      return line;
    }
  }
  
  Serial.println();
  Serial.println("closing connection");
  return "";
  }

//============================================================== getValue
String getValue(int memoryIndex, String* responseData){
   String s= "&V"+String(memoryIndex)+'=';
   int index = responseData->indexOf(s);
   if (index!=-1){
     int nextCommandIndex= responseData->indexOf('&',index+1);
     if (nextCommandIndex==-1) return responseData->substring(index+s.length());
     else return responseData->substring(index+s.length(),nextCommandIndex);
   }  
   return "";
}


//----------------------------------------------------------------------------
/* Virtuino GET request to upload values of V1 and V4 memories to board  (KEY=1234)
 *  GET /&KEY=1234&V1=12.3&V4=1
 *  Response: OK 
 *  
 * Virtuino GET request to read values from board (KEY=1234)
 *  GET /&KEY=1234&V2=?&V3=?
 *  Response: &V2=123&V3=1
 *  
 *  * Virtuino GET request to upload and read values from board (KEY=1234)
 *  GET /&KEY=1234&V2=45.6&V3=?
 *  Response: &V3=1
 *  
 *  Virtuino GET request to check the connection  (KEY=1234)
 *  GET /&KEY=1234&CNT=?
 *  Response: &CNT=Hello Virtuino I am the ESP8266 board
 *  
 * Virtuino GET request to upload value as TEXT from memory V1 to board  (KEY=1234)
 *  GET /&KEY=1234&T1=Hello
 *  Response: OK 

 * Virtuino GET request to read value a TEXT from board  (KEY=1234)
 *  GET /&KEY=1234&T1=?
 *  Response &T1=the text you want to send 
 *  or
 *  GET /&KEY=1234&V1=?
 *  Response &V1=the text you want to send 
 *  
 *  All the texts that are sent from Virtuino are encoded 
 *  All the texts that received from Virtuino will be decoded
 */



